#!/usr/bin/env python
"""
DS2490 OneWire master for the pyonewire package

  Copyright 2008 mike wakerly <opensource@hoho.com>

Portions of this file have been derived from the Linux kernel files w1.h, w1.c,
and ds2490.c, which have the following copyright notices:

  Copyright (c) 2004 Evgeniy Polyakov <johnpol@2ka.mipt.ru>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
"""

import logging
import time
import usb

from pyonewire.core import cstruct
from pyonewire.core import util
from pyonewire.master import GenericOneWireMaster

# Request codes
CONTROL_CMD = 0x00
COMM_CMD = 0x01
MODE_CMD = 0x02
TEST_CMD = 0x03

# Status flags
ST_SPUA = 0x01  # strong pull-up is active
ST_PRGA = 0x02  # 12v programming pulse is being generated
ST_12VP = 0x04  # external 12v programming voltage is present
ST_PMOD = 0x08  # ds2490 powered from USB and external sources
ST_HALT = 0x10  # ds2490 is currently halted
ST_IDLE = 0x20  # ds2490 is currently idle
ST_EPOF = 0x80

# Result Register flags
RR_DETECT = 0xA5  # new device detected
RR_NRS = 0x01  # no response to search
RR_SH = 0x02  # short on reset or set path
RR_APP = 0x04  # alarming presence on reset
RR_VPP = 0x08  # 12v expected not seen
RR_CMP = 0x10  # compare error
RR_CRC = 0x20  # crc error detected
RR_RDP = 0x40  # redirected page
RR_EOS = 0x80  # end of search error

# Value field, Control commands
# Control Command Code Constants
CTL_RESET_DEVICE = 0x0000
CTL_START_EXE = 0x0001
CTL_RESUME_EXE = 0x0002
CTL_HALT_EXE_IDLE = 0x0003
CTL_HALT_EXE_DONE = 0x0004
CTL_CANCEL_CMD = 0x0005
CTL_CANCEL_MACRO = 0x0006
CTL_FLUSH_COMM_CMDS = 0x0007
CTL_FLUSH_RCV_BUFFER = 0x0008
CTL_FLUSH_XMT_BUFFER = 0x0009
CTL_GET_COMM_CMDS = 0x000A

# USB endpoints
EP_CONTROL = 0
EP_STATUS = 1
EP_DATA_OUT = 2
EP_DATA_IN = 3

# Value field COMM Command options
# Byte 1
COMM_TYPE = 0x0008
COMM_SE = 0x0008
COMM_D = 0x0008
COMM_Z = 0x0008
COMM_CH = 0x0008
COMM_SM = 0x0008
COMM_R = 0x0008
COMM_IM = 0x0001

# Byte 2
COMM_PS = 0x4000
COMM_PST = 0x4000
COMM_CIB = 0x4000
COMM_RTS = 0x4000
COMM_DT = 0x2000
COMM_SPU = 0x1000
COMM_F = 0x0800
COMM_ICP = 0x0200
COMM_RST = 0x0100

# Value field COMM Command options (0-F plus assorted bits)
COMM_ERROR_ESCAPE = 0x0601
COMM_SET_DURATION = 0x0012
COMM_BIT_IO = 0x0020
COMM_PULSE = 0x0030
COMM_1_WIRE_RESET = 0x0042
COMM_BYTE_IO = 0x0052
COMM_MATCH_ACCESS = 0x0064
COMM_BLOCK_IO = 0x0074
COMM_READ_STRAIGHT = 0x0080
COMM_DO_RELEASE = 0x6092
COMM_SET_PATH = 0x00A2
COMM_WRITE_SRAM_PAGE = 0x00B2
COMM_WRITE_EPROM = 0x00C4
COMM_READ_CRC_PROT_PAGE = 0x00D4
COMM_READ_REDIRECT_PAGE_CRC = 0x21E4
COMM_SEARCH_ACCESS = 0x00F4

# Mode Command Code Constants 
# Enable Pulse Constants
ENABLEPULSE_PRGE = 0x01  # strong pull-up
ENABLEPULSE_SPUE = 0x02  # programming pulse

# 1Wire Bus Speed Setting Constants
ONEWIREBUSSPEED_REGULAR = 0x00
ONEWIREBUSSPEED_FLEXIBLE = 0x01
ONEWIREBUSSPEED_OVERDRIVE = 0x02

# Value field Mode Commands options
MOD_PULSE_EN = 0x0000
MOD_SPEED_CHANGE_EN = 0x0001
MOD_1WIRE_SPEED = 0x0002
MOD_STRONG_PU_DURATION = 0x0003
MOD_PULLDOWN_SLEWRATE = 0x0004
MOD_PROG_PULSE_DURATION = 0x0005
MOD_WRITE1_LOWTIME = 0x0006
MOD_DSOW0_TREC = 0x0007

# duration of strong pullup, in ms
PULLUP_PULSE_DURATION = 750

TIMEOUT_LIBUSB = 1000

TRACE = False
TRACE_LEVEL = 0

def trace(fn):
  def wrapped(*args, **kwargs):
    global TRACE_LEVEL
    if not TRACE:
      return fn(*args, **kwargs)
    pad = '  ' * TRACE_LEVEL
    TRACE_LEVEL += 1
    print '%s> %s' % (pad, fn.__name__)
    ret = fn(*args, **kwargs)
    print '%s< %s' % (pad, fn.__name__)
    TRACE_LEVEL -= 1
    return ret
  return wrapped


class UsbError(Exception):
  """Raised when a libusb operation returns an error"""

def StatusPacket():
  return cstruct.cStruct(( ('B', 'EnableFlags'),
                           ('B', 'OneWireSpeed'),
                           ('B', 'StrongPullUpDirection'),
                           ('B', 'ProgPulseDuration'),
                           ('B', 'PullDownSlewRate'),
                           ('B', 'Write1LowTime'),
                           ('B', 'DSOW0RecoveryTime'),
                           ('B', 'Reserved1'),
                           ('B', 'StatusFlags'),
                           ('B', 'CurrentCommCmd1'),
                           ('B', 'CurrentCommCmd2'),
                           ('B', 'CommBufferStatus'),
                           ('B', 'WriteBufferStatus'),
                           ('B', 'ReadBufferStatus'),
                           ('B', 'Reserved2'),
                           ('B', 'Reserved3'),
                        ))


def GetDevice(vendor_id, product_id):
  buses = usb.busses()
  for bus in buses:
    for device in bus.devices:
      if (device.idVendor, device.idProduct) == (vendor_id, product_id):
        return device
  return None

logging.basicConfig(level=logging.DEBUG)

class DS2490Master(GenericOneWireMaster.GenericOneWireMaster):
  VENDORID    = 0x04fa
  PRODUCTID   = 0x2490
  INTERFACEID = 0
  SEARCH_NORMAL = 0xf0
  SEARCH_ALARM = 0xec
  def __init__(self):
    GenericOneWireMaster.GenericOneWireMaster.__init__(self)
    self._logger = logging.getLogger("ds2490")
    self._device = None
    self._handle = None

    self._device = GetDevice(self.VENDORID, self.PRODUCTID)

    if not self._device:
      self._logger.fatal('Could not acquire device')
      raise RuntimeError, "no device"

    self._handle = self._device.open()
    self._handle.reset()

    self._conf = self._device.configurations[0]
    self._intf = self._conf.interfaces[0][0]
    self._handle.setConfiguration(self._conf)
    self._handle.claimInterface(self._intf)
    self._handle.setAltInterface(1)

    self.Reset()

    self._logger.debug('completed init')

  def _SendMessage(self, command, value, index, timeout=TIMEOUT_LIBUSB):
    ret = self._handle.controlMsg(requestType=0x40, request=command, buffer='',
                                  value=value, index=index, timeout=timeout)
    if ret:
      raise UsbError, "Error while sending control message: %s" % (ret,)

  @trace
  def SendControlCommand(self, value, index, timeout=TIMEOUT_LIBUSB):
    self._SendMessage(CONTROL_CMD, value, index, timeout)

  @trace
  def SendControlMode(self, value, index, timeout=TIMEOUT_LIBUSB):
    self._SendMessage(MODE_CMD, value, index, timeout)

  @trace
  def SendControl(self, value, index, timeout=TIMEOUT_LIBUSB):
    self._SendMessage(COMM_CMD, value, index, timeout)

  @trace
  def GetStatus(self):
    raw = self._handle.bulkRead(EP_STATUS, 32, TIMEOUT_LIBUSB)
    status = StatusPacket()
    status.UnpackFromTuple(raw[:16])
    result_regs = raw[16:]
    if result_regs:
      self._logger.info('result regs: %s' % repr(result_regs))
    if status.StatusFlags & ST_EPOF:
      self._logger.info('Resetting device after ST_EPOF')
      self.SendControlCommand(CTL_RESET_DEVICE, 0)
    return status, result_regs

  @trace
  def RecvData(self, size):
    try:
      raw = self._handle.bulkRead(EP_DATA_IN, size, TIMEOUT_LIBUSB)
    except usb.USBError:
      print self.WaitStatus()
      raise
    return raw

  @trace
  def SendData(self, buf):
    return self._handle.bulkWrite(EP_DATA_OUT, buf, TIMEOUT_LIBUSB)

  @trace
  def WaitStatus(self):
    count = 0
    status = None
    while True:
      status, regs = self.GetStatus()
      count += 1
      time.sleep(0.01)
      if status.StatusFlags & ST_IDLE:
        break
      if count >= 100:
        raise RuntimeError, "took too long to get status"
    return status

  @trace
  def Reset(self):
    # por reset
    self.SendControlCommand(value=CTL_RESET_DEVICE, index=0)

    # set speed
    self.SendControl(0x43, ONEWIREBUSSPEED_FLEXIBLE)

    # set the strong pullup duration to infinite
    self.SendControl(value=(COMM_SET_DURATION | COMM_IM), index=0)

    # set the 12V pullup duration to 512us
    self.SendControl(value=(COMM_SET_DURATION | COMM_IM | COMM_TYPE),
                     index=0x40)

    # disable strong pullup, but leave progrm pulse enabled (faster)
    self.SendControlMode(value=MOD_PULSE_EN, index=ENABLEPULSE_PRGE)

    return self.WaitStatus()

  @trace
  def StartPulse(self, delay):
    outdelay = 1 + (delay >> 4)
    self.SendControlMode(MOD_PULSE_EN, ENABLEPULSE_SPUE)
    self.SendControl(COMM_SET_DURATION | COMM_IM, outdelay)
    self.SendControl(COMM_PULSE | COMM_IM | COMM_F, 0)
    time.sleep(delay/1000.0) # mdelay
    return self.WaitStatus()

  @trace
  def TouchBit(self, bit):
    val = COMM_BIT_IO | COMM_IM
    if bit:
      val |= COMM_D

    self.SendControl(val, 0)
    count = 0
    while True:
      count += 1
      status = self.WaitStatus()
      cmd = status.CurrentCommCmd1 | (status.CurrentCommCmd2 << 8)
      if (cmd == val) or count >= 10:
        break
    if count >= 10:
      raise ValueError, "Too many tries"

    ret = self.RecvData(1)
    return ret[0] 

  @trace
  def WriteBit(self, bit):
    val = COMM_BIT_IO | COMM_IM | COMM_ICP
    if bit:
      val |= COMM_D
    self.SendControl(val)

    return self.WaitStatus()

  @trace
  def WriteByte(self, byte):
    self.SendControl(COMM_BYTE_IO | COMM_IM | COMM_SPU, byte)
    st = self.WaitStatus()
    rbyte = self.RecvData(1)
    self.StartPulse(PULLUP_PULSE_DURATION)
    if len(rbyte) == 1:
      return byte != rbyte[0]
    else:
      return False

  @trace
  def ReadByte(self):
    self.SendControl(COMM_BYTE_IO | COMM_IM, 0xff)
    self.WaitStatus()
    ret = self.RecvData(1)
    return ret[0]

  @trace
  def ReadBlock(self, blocklen):
    if blocklen > (64*1024):
      raise ValueError, "Len too long"

    buf = [0xff]*blocklen
    self.SendData(buf)
    self.SendControl(COMM_BLOCK_IO | COMM_IM | COMM_SPU, blocklen)
    self.WaitStatus()

    return self.RecvData(blocklen)

  @trace
  def WriteBlock(self, buf):
    self.SendData(buf)
    self.SendControl(COMM_BLOCK_IO | COMM_IM | COMM_SPU, len(buf))
    self.WaitStatus()
    b2 = self.RecvData(len(buf))
    self.StartPulse(PULLUP_PULSE_DURATION)

    return len(b2) != len(buf)

  @trace
  def Search(self, search_type=SEARCH_NORMAL, max=0):
    self.Reset()

    # Hardware search command is not working properly when more than one device
    # is attached.
    if True:
      return GenericOneWireMaster.GenericOneWireMaster.Search(self, search_type)

    self.SendData('\x00'*8)
    self.WaitStatus()

    val = COMM_SEARCH_ACCESS | COMM_IM | COMM_SM | COMM_F
    index = (max << 8) | search_type
    self.SendControl(val, index)

    ret = []
    count = 0
    last = False

    # The read data endpoint has a maximum size of 16 bytes. To support reads
    # of 2 or more ibuttons, we need to pull from it while the search command
    # is underway.
    ids = []
    status = None
    while True:
      while len(ret) >= 8:
        ids.append(util.IdTupleToLong(ret[:8]))
        ret = ret[8:]
      if last:
        break
      ret += list(self.RecvData(8))
      status, regs = self.GetStatus()
      if status.StatusFlags & ST_IDLE:
        last = True
      for val in regs:
        if val == RR_DETECT:
          self._logger.info('new device detected')
          continue
        elif (val & RR_NRS) or (val & RR_EOS):
          self._logger.warning('NRS error')
          last = True
      time.sleep(0.01)
      count += 1
      if count >= 100:
        raise RuntimeError, "took too long to get status"

    return ibs


def mkserial(num):
  return ' '.join(['%02x' % ((num >> (8*i)) & 0xff) for i in range(8)])


if __name__ == '__main__':
  dev = DS2490Master()
  while True:
    printed = False
    for d in dev.Search(dev.SEARCH_NORMAL):
      print hex(d)
      printed = True
    if printed:
      print ''



"""
cstruct.py
by mike wakerly - http://hoho.com/mike/
"""

import struct

class cStruct:
   """
   A class to represent C structs in python.

   This class provides an interface to the python struct module. Structs
   can be defined by field, with their associated struct format character. A
   simple example:

      p = cStruct( ('h', 'shortid'), ('l', 'longnum') )
      p.shortid = 2
      p.longnum = 10e6
      print repr(p.pack())
   """
   def __init__(self, fields):
      _fields  = []
      _formats = {}
      _values  = {}

      for f in fields:
         _fields.append(f[1])
         _formats[f[1]] = f[0]
         if len(f) == 3:
            _values[f[1]] = f[2]
         else:
            _values[f[1]] = 0

      self.__dict__['_fields']  = _fields
      self.__dict__['_formats'] = _formats
      self.__dict__['_values']  = _values

   def __str__(self):
      return '<cStruct: %s>' % ', '.join(['%s=%s' % x for x in (zip(self._fields, self._values.values()))])

   def __getattr__(self, a):
      return self._values.get(a)

   def __setattr__(self, a, val):
      self._values[a] = val

   def __eq__(self, other):
      return self._values == other._values

   def UnpackFromTuple(self, t):
     assert len(t) == len(self._fields)
     for i in xrange(len(self._fields)):
       self._values[self._fields[i]] = t[i]

   def unpack(self, datastr):
      pos = 0
      if len(datastr) == 0:
         return
      for f in self._fields:
         if pos > len(datastr):
            break
         fmt = self._formats[f]
         if fmt == 's':
            #print 'unpack %s AS STR' % f
            datalen = len(datastr) - pos # XXX string must be last in struct
            setattr(self, f, struct.unpack('%is' % datalen, datastr[pos:pos+datalen])[0])
         else:
            #print 'unpack %s' % f
            datalen = struct.calcsize(fmt)
            setattr(self, f, struct.unpack(fmt, datastr[pos:pos+datalen])[0])
         #print 'size %i' % datalen
         pos += datalen


   def pack(self):
      ret = ''
      for f in self._fields:
         if self._formats[f] == 's':
            if self._values[f]: #XXX - no string = no data
               ret += struct.pack('%is' % len(self._values[f]), self._values[f])
         else:
            ret += struct.pack(self._formats[f], self._values[f])
      return ret



def IdTupleToLong(v):
  if len(v) != 8:
    raise ValueError, "Tuple must consist of 8 integers"
  ret = 0L
  for i in xrange(8):
    # this reverses the order of the tuple
    ret |= (v[i] << (i*8))
  return ret



#!/usr/bin/env python

from distutils.core import setup

setup(
    name = "pyonewire",
    version = "0.5.1",
    description = "Python interface to 1-Wire devices",
    author = "mike wakerly",
    author_email = "opensource@hoho.com",
    url = "http://code.google.com/p/pyonewire/",
    packages = [
      'pyonewire',
      'pyonewire.core',
      'pyonewire.master',
    ],
    package_dir = {
      'pyonewire': '',
    },
)



#!/usr/bin/env python
"""
Generic OneWire master for the pyonewire package

  Copyright 2008 mike wakerly <opensource@hoho.com>

Portions of this file have been derived from the Linux kernel files w1.h, w1.c,
and ds2490.c, which have the following copyright notices:

  Copyright (c) 2004 Evgeniy Polyakov <johnpol@2ka.mipt.ru>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
"""

class GenericOneWireMaster(object):
  """Base class for onewire bus masters.

  This class is derived from the linux kernel struct w1_bus_master.
  """
  def Reset(self):
    """Clear any device state"""
    raise NotImplementedError

  def StartPulse(self, delay):
    """Issue a onewire start pulse"""
    raise NotImplementedError

  def ReadBit(self):
    """Sample the line level.

    Returns
      The level read (0 or 1)
    """
    raise NotImplementedError

  def WriteBit(self, bit):
    """Set the bus line level.

    Args
      bit - level to set
    Returns
      None
    """
    raise NotImplementedError

  def TouchBit(self, bit):
    """Low-level onewire function.

    If bit is clear, causes a onewire "write 0" cycle. If set, causes a onewire
    "write 1" or read cycle.

    Args
      bit - write 0 if clear, write 1 or read if set
    Returns
      the bit read (0 or 1)
    """
    raise NotImplementedError

  def ReadByte(self):
    """Read a byte from the bus.

    Equivalent to 8 TouchBit(1) calls.

    Returns:
      the value read
    """
    raise NotImplementedError

  def WriteByte(self, byte):
    """Write an entire byte to the bus.

    Equivalent to 8 TouchBit(x) calls.

    Args
      byte - value to write
    Returns
      None
    """
    raise NotImplementedError

  def ReadBlock(self, numblocks):
    """Read a series of bytes form the bus.

    Equivalent to |numblocks| ReadByte calls.

    Args
      numblocks - number of bytes to read
    Returns
      iterable of bytes returned
    """
    raise NotImplementedError

  def WriteBlock(self, data):
    """Write several bytes to the bus.

    Equivalent to len(data) WriteByte calls.

    Args
      data - iterable of integers as bytes to write
    Returns
      None
    """
    raise NotImplementedError

  def Triplet(self, bdir):
    """Combination of two reads and smart write for ROM search.

    Args
      dbit - direction to choose if both are valid
    Returns
      tuple of (id_bit, comp_bit, dir_taken)
    """
    id_bit = self.TouchBit(1)
    comp_bit = self.TouchBit(1)

    if id_bit and comp_bit:
      return 0x03 # error

    if not id_bit and not comp_bit:
      # both bits are valid, take the direction given
      if bdir:
        retval = 0x04
      else:
        retval = 0
    else:
      # only one bit is valid, take that direction
      bdir = id_bit
      if id_bit:
        retval = 0x05
      else:
        retval = 0x02

    self.TouchBit(bdir)

    return retval

  def Search(self, search_type):
    """Generator that performs search and yields address found.

    Args
      search_type - one of SEARCH_NORMAL, SEARCH_ALARM
    Yields
      64 bit integer ids found
    """
    last_rn = 0L
    rn = 0L
    tmp64 = 0L
    i = 0L
    last_device = 0L
    last_zero = -1L

    desc_bit = 64

    ret = []

    def pick_search_bit(i, desc_bit, last_rn):
      if i == desc_bit:
        # took 0 path last time, so take 1 path
        return 1
      elif i > desc_bit:
        # take the 0 path on the next branch
        return 0
      else:
        return ((last_rn >> i) & 0x1)

    while not last_device:
      last_rn = rn
      rn = 0L

      # Reset bus
      self.Reset()

      self.WriteByte(search_type)

      for i in xrange(64):
        # Determine the direction/search bit
        search_bit = pick_search_bit(i, desc_bit, last_rn)

        # read two bits and write one bit
        triplet_ret = self.Triplet(search_bit)

        if (triplet_ret & 0x03) == 0x03:
          # quit if no devices responded
          break

        # if both directions were valid & we took the 0 path
        if triplet_ret == 0:
          last_zero = i

        # extract the direction taken & update the device number
        tmp64 = triplet_ret >> 2
        rn |= (tmp64 << i)

      if triplet_ret & 0x03 != 0x03:
        if desc_bit == last_zero or last_zero < 0:
          last_device = True
        desc_bit = last_zero
        ret.append(rn)

    return ret
